@@ -113,7 +113,7 @@ raptor_ntriples_parse_terminate(raptor_parser* rdf_parser)
 }
 
 
-static const char *term_type_strings[]={
+static const char * const term_type_strings[]={
   "URIref",
   "bnodeID",
   "Literal"
@@ -749,7 +749,9 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
             goto cleanup;
           }
           strcpy((char*)blank, (const char*)dest);
-          dest=raptor_parser_internal_generate_id(rdf_parser, 0, blank);
+          dest=raptor_parser_internal_generate_id(rdf_parser, 
+                                                  RAPTOR_GENID_TYPE_BNODEID,
+                                                  blank);
           terms_allocated[i]=1;
         }
 
@@ -1111,9 +1113,11 @@ raptor_ntriples_parse_recognise_syntax(raptor_parser_factory* factory,
 }
 
 
-static void
+static int
 raptor_ntriples_parser_register_factory(raptor_parser_factory *factory) 
 {
+  int rc=0;
+
   factory->context_length     = sizeof(raptor_ntriples_parser_context);
 
   factory->need_base_uri = 0;
@@ -1124,14 +1128,16 @@ raptor_ntriples_parser_register_factory(raptor_parser_factory *factory)
   factory->chunk     = raptor_ntriples_parse_chunk;
   factory->recognise_syntax = raptor_ntriples_parse_recognise_syntax;
 
-  raptor_parser_factory_add_uri(factory, 
+  rc=raptor_parser_factory_add_uri(factory, 
                                 (const unsigned char*)"http://www.w3.org/TR/rdf-testcases/#ntriples");
+
+  return rc;
 }
 
 
-void
+int
 raptor_init_parser_ntriples(void)
 {
-  raptor_parser_register_factory("ntriples",  "N-Triples",
-                                 &raptor_ntriples_parser_register_factory);
+  return !raptor_parser_register_factory("ntriples",  "N-Triples",
+                                         &raptor_ntriples_parser_register_factory);
 }
