@@ -2,7 +2,7 @@
  *
  * raptor_sax2.c - Raptor SAX2 API
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -52,12 +52,13 @@
 #undef RAPTOR_DEBUG_CDATA
 
 
-void
+int
 raptor_sax2_init(void)
 {
 #ifdef RAPTOR_XML_LIBXML
   xmlInitParser();
 #endif
+  return 0;
 }
 
 
@@ -66,15 +67,23 @@ raptor_sax2_finish(void)
 {
 #ifdef RAPTOR_XML_LIBXML
   xmlCleanupParser();
+  xmlSetStructuredErrorFunc(NULL, NULL);
+  xmlSetGenericErrorFunc(NULL, NULL);
 #endif
 }
 
 
+/**
+ * raptor_new_sax2:
+ * @user_data: pointer context information to pass to handlers
+ * @error_handlers: error handlers pointer
+ *
+ * Constructor - Create a new SAX2 with error handlers
+ *
+ * Return value: new #raptor_sax2 object or NULL on failure
+ */
 raptor_sax2*
-raptor_new_sax2(void* user_data,
-                void *error_data, raptor_message_handler error_handler,
-                void *fatal_error_data, raptor_message_handler fatal_error_handler,
-                void *warning_data, raptor_message_handler warning_handler)
+raptor_new_sax2(void* user_data, raptor_error_handlers* error_handlers)
 {
   raptor_sax2* sax2;
   sax2=(raptor_sax2*)RAPTOR_CALLOC(raptor_sax2, 1, sizeof(raptor_sax2));
@@ -84,19 +93,32 @@ raptor_new_sax2(void* user_data,
 #ifdef RAPTOR_XML_LIBXML
   sax2->magic=RAPTOR_LIBXML_MAGIC;
 #endif
-  
+
   sax2->user_data=user_data;
-  sax2->error_data=error_data;
-  sax2->error_handler=error_handler;
-  sax2->fatal_error_data=fatal_error_data;
-  sax2->fatal_error_handler=fatal_error_handler;
-  sax2->warning_data=warning_data;
-  sax2->warning_handler=warning_handler;
+
+  sax2->locator=error_handlers->locator;
+  
+  sax2->error_handlers=error_handlers;
+
+#ifdef RAPTOR_XML_LIBXML
+  /* sets xmlGenericErrorContext and xmlStructuredError */
+  xmlSetStructuredErrorFunc(&sax2->error_handlers, 
+                            raptor_libxml_xmlStructuredErrorFunc);
+  /* sets xmlGenericErrorContext and xmlGenericError */
+  xmlSetGenericErrorFunc(&sax2->error_handlers, 
+                         (xmlGenericErrorFunc)raptor_libxml_generic_error);
+#endif
   
   return sax2;
 }
 
 
+/**
+ * raptor_free_sax2:
+ * @sax2: SAX2 object
+ *
+ * Destructor - destroy a SAX2 object
+ */
 void
 raptor_free_sax2(raptor_sax2 *sax2) {
   raptor_xml_element *xml_element;
@@ -127,7 +149,13 @@ raptor_free_sax2(raptor_sax2 *sax2) {
 }
 
 
-
+/**
+ * raptor_sax2_set_start_element_handler:
+ * @sax2: SAX2 object
+ * @handler: start element handler
+ *
+ * Set SAX2 start element handler.
+ */
 void
 raptor_sax2_set_start_element_handler(raptor_sax2* sax2,
                                       raptor_sax2_start_element_handler handler)
@@ -136,6 +164,13 @@ raptor_sax2_set_start_element_handler(raptor_sax2* sax2,
 }
 
 
+/**
+ * raptor_sax2_set_end_element_handler:
+ * @sax2: SAX2 object
+ * @handler: end element handler
+ *
+ * Set SAX2 end element handler.
+ */
 void
 raptor_sax2_set_end_element_handler(raptor_sax2* sax2,
                                     raptor_sax2_end_element_handler handler)
@@ -144,6 +179,13 @@ raptor_sax2_set_end_element_handler(raptor_sax2* sax2,
 }
 
 
+/**
+ * raptor_sax2_set_characters_handler:
+ * @sax2: SAX2 object
+ * @handler: characters handler
+ *
+ * Set SAX2 characters handler.
+ */
 void
 raptor_sax2_set_characters_handler(raptor_sax2* sax2,
                                    raptor_sax2_characters_handler handler)
@@ -152,6 +194,13 @@ raptor_sax2_set_characters_handler(raptor_sax2* sax2,
 }
 
 
+/**
+ * raptor_sax2_set_cdata_handler:
+ * @sax2: SAX2 object
+ * @handler: CDATA handler
+ *
+ * Set SAX2 CDATA handler.
+ */
 void
 raptor_sax2_set_cdata_handler(raptor_sax2* sax2,
                               raptor_sax2_cdata_handler handler)
@@ -160,6 +209,13 @@ raptor_sax2_set_cdata_handler(raptor_sax2* sax2,
 }
 
 
+/**
+ * raptor_sax2_set_comment_handler:
+ * @sax2: SAX2 object
+ * @handler: comment handler
+ *
+ * Set SAX2 XML comment handler.
+ */
 void
 raptor_sax2_set_comment_handler(raptor_sax2* sax2,
                                 raptor_sax2_comment_handler handler)
@@ -168,6 +224,13 @@ raptor_sax2_set_comment_handler(raptor_sax2* sax2,
 }
 
 
+/**
+ * raptor_sax2_set_unparsed_entity_decl_handler:
+ * @sax2: SAX2 object
+ * @handler: unparsed entity declaration handler
+ *
+ * Set SAX2 XML unparsed entity declaration handler.
+ */
 void
 raptor_sax2_set_unparsed_entity_decl_handler(raptor_sax2* sax2,
                                              raptor_sax2_unparsed_entity_decl_handler handler)
@@ -176,6 +239,13 @@ raptor_sax2_set_unparsed_entity_decl_handler(raptor_sax2* sax2,
 }
 
 
+/**
+ * raptor_sax2_set_external_entity_ref_handler:
+ * @sax2: SAX2 object
+ * @handler: entity reference handler
+ *
+ * Set SAX2 XML entity reference handler.
+ */
 void
 raptor_sax2_set_external_entity_ref_handler(raptor_sax2* sax2,
                                             raptor_sax2_external_entity_ref_handler handler)
@@ -186,8 +256,7 @@ raptor_sax2_set_external_entity_ref_handler(raptor_sax2* sax2,
 
 /**
  * raptor_sax2_set_namespace_handler:
- * @parser: #raptor_sax2 object
- * @user_data: user data pointer for callback
+ * @sax2: #raptor_sax2 object
  * @handler: new namespace callback function
  *
  * Set the XML namespace handler function.
@@ -200,7 +269,7 @@ raptor_sax2_set_external_entity_ref_handler(raptor_sax2* sax2,
  * namespace may be declared multiple times when a namespace is seen
  * in different parts of a document.
  * 
- **/
+ */
 void
 raptor_sax2_set_namespace_handler(raptor_sax2* sax2,
                                   raptor_namespace_handler handler)
@@ -209,13 +278,6 @@ raptor_sax2_set_namespace_handler(raptor_sax2* sax2,
 }
 
 
-void
-raptor_sax2_set_locator(raptor_sax2* sax2, raptor_locator* locator)
-{
-  sax2->locator=locator;
-}
-
-
 raptor_xml_element*
 raptor_xml_element_pop(raptor_sax2 *sax2) 
 {
@@ -249,7 +311,7 @@ raptor_xml_element_push(raptor_sax2 *sax2, raptor_xml_element* element)
  * Check if an XML Element is empty.
  * 
  * Return value: non-0 if the element is empty.
- **/
+ */
 int
 raptor_xml_element_is_empty(raptor_xml_element* xml_element)
 {
@@ -258,6 +320,14 @@ raptor_xml_element_is_empty(raptor_xml_element* xml_element)
 }
 
 
+/**
+ * raptor_sax2_inscope_xml_language:
+ * @sax2: SAX2 object
+ * 
+ * Get the in-scope XML language
+ * 
+ * Return value: the XML language or NULL if none is in scope.
+ */
 const unsigned char*
 raptor_sax2_inscope_xml_language(raptor_sax2 *sax2)
 {
@@ -266,13 +336,25 @@ raptor_sax2_inscope_xml_language(raptor_sax2 *sax2)
   for(xml_element=sax2->current_element;
       xml_element; 
       xml_element=xml_element->parent)
-    if(xml_element->xml_language)
+    if(xml_element->xml_language) {
+      if(!*xml_element->xml_language)
+        return NULL;
       return xml_element->xml_language;
+    }
+  
     
   return NULL;
 }
 
 
+/**
+ * raptor_sax2_inscope_base_uri:
+ * @sax2: SAX2 object
+ * 
+ * Get the in-scope base URI
+ * 
+ * Return value: the in-scope base URI shared object or NULL if none is in scope.
+ */
 raptor_uri*
 raptor_sax2_inscope_base_uri(raptor_sax2 *sax2)
 {
@@ -323,21 +405,31 @@ raptor_sax2_simple_error(void* user_data, const char *message, ...)
 
   va_start(arguments, message);
 
-  if(sax2)
-    raptor_invoke_message_varargs("error",
-                                  sax2->error_handler,
-                                  sax2->error_data,
-                                  sax2->locator,
-                                  message, arguments);
+  if(sax2) {
+    raptor_log_level level=RAPTOR_LOG_LEVEL_ERROR;
+    raptor_message_handler_closure* cl;
+    cl=&sax2->error_handlers->handlers[level];
+    raptor_log_error_varargs(level, cl->handler, cl->user_data,
+                             sax2->locator,
+                             message, arguments);
+  }
   
   va_end(arguments);
 }
 
 
+
+/**
+ * raptor_sax2_parse_start:
+ * @sax2: sax2 object
+ * @base_uri: base URI
+ *
+ * Start an XML SAX2 parse.
+ */
 void
 raptor_sax2_parse_start(raptor_sax2* sax2, raptor_uri *base_uri)
 {
-  raptor_uri_handler *uri_handler;
+  const raptor_uri_handler *uri_handler;
   void *uri_context;
 
   sax2->depth=0;
@@ -363,6 +455,9 @@ raptor_sax2_parse_start(raptor_sax2* sax2, raptor_uri *base_uri)
 #ifdef RAPTOR_XML_LIBXML
   raptor_libxml_init(sax2, base_uri);
 
+  xmlSetStructuredErrorFunc(sax2->error_handlers, 
+                            raptor_libxml_xmlStructuredErrorFunc);
+
 #if LIBXML_VERSION < 20425
   sax2->first_read=1;
 #endif
@@ -384,18 +479,30 @@ raptor_sax2_parse_start(raptor_sax2* sax2, raptor_uri *base_uri)
 }
 
 
+/**
+ * raptor_sax2_parse_chunk:
+ * @sax2: sax2 object
+ * @buffer: input buffer
+ * @len: input buffer lenght
+ * @is_end: non-0 if end of data
+ *
+ * Parse a chunk of XML data generating SAX2 events
+ *
+ * Return value: non-0 on failure
+ */
 int
 raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
                         size_t len, int is_end) 
 {
 #ifdef RAPTOR_XML_EXPAT
   XML_Parser xp=sax2->xp;
+  int rc;
 #endif
 #ifdef RAPTOR_XML_LIBXML
   /* parser context */
   xmlParserCtxtPtr xc=sax2->xc;
-#endif
   int rc;
+#endif
   
 #ifdef RAPTOR_XML_LIBXML
   if(!xc) {
@@ -404,8 +511,9 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
     if(!len) {
       /* no data given at all - emit a similar message to expat */
       raptor_sax2_update_document_locator(sax2, sax2->locator);
-      sax2->error_handler(sax2->error_data, sax2->locator,
-                          "XML Parsing failed - no element found");
+      raptor_log_error_to_handlers(sax2->error_handlers,
+                                   RAPTOR_LOG_LEVEL_ERROR, sax2->locator,
+                                   "XML Parsing failed - no element found");
       return 1;
     }
 
@@ -415,7 +523,7 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
     if(!xc)
       goto handle_error;
 
-#ifdef XML_PARSE_NONET
+#ifdef RAPTOR_LIBXML_XML_PARSE_NONET
     if(sax2->feature_no_net)
       libxml_options |= XML_PARSE_NONET;
 #endif
@@ -500,7 +608,9 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
 
   return 0;
 
+#if defined(RAPTOR_XML_EXPAT) || defined(RAPTOR_XML_LIBXML)
   handle_error:
+#endif
 
 #ifdef RAPTOR_XML_EXPAT
 #ifdef EXPAT_UTF8_BOM_CRASH
@@ -532,15 +642,16 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
       strncpy(error_buffer, error_prefix, ERROR_PREFIX_LEN);
       strncpy(error_buffer+ERROR_PREFIX_LEN, error_message, error_length+1);
 
-      sax2->error_handler(sax2->error_data, sax2->locator, error_buffer);
+      raptor_log_error_to_handlers(sax2->error_handlers,
+                                   RAPTOR_LOG_LEVEL_ERROR,
+                                   sax2->locator, error_buffer);
       RAPTOR_FREE(cstring, error_buffer);
     } else
-      sax2->error_handler(sax2->error_data, sax2->locator, "XML Parsing failed");
+      raptor_log_error_to_handlers(sax2->error_handlers,
+                                   RAPTOR_LOG_LEVEL_ERROR,
+                                   sax2->locator, "XML Parsing failed");
   }
 #endif
-#ifdef RAPTOR_XML_LIBXML
-  sax2->error_handler(sax2->error_data, sax2->locator, "XML Parsing failed");
-#endif
 
   return 1;
 }
@@ -557,7 +668,7 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
  * The allowed features are available via raptor_sax2_features_enumerate().
  *
  * Return value: non 0 on failure or if the feature is unknown
- **/
+ */
 int
 raptor_sax2_set_feature(raptor_sax2 *sax2, raptor_feature feature, int value)
 {
@@ -582,6 +693,10 @@ raptor_sax2_set_feature(raptor_sax2 *sax2, raptor_feature feature, int value)
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
     case RAPTOR_FEATURE_RELATIVE_URIS:
     case RAPTOR_FEATURE_START_URI:
     case RAPTOR_FEATURE_WRITER_AUTO_INDENT:
@@ -598,6 +713,17 @@ raptor_sax2_set_feature(raptor_sax2 *sax2, raptor_feature feature, int value)
     case RAPTOR_FEATURE_LITERAL_FILL:
     case RAPTOR_FEATURE_BNODE_FILL:
 
+    /* JSON serializer features */
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    
+    /* Turtle serializer feature */
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
+
+    /* WWW feature */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+      
     default:
       return -1;
       break;
@@ -652,9 +778,11 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
       unsigned char *src = value;
       unsigned char *dst = xmlStrdup(value);
 
-      if (!dst) {
-        sax2->error_handler(sax2->error_data, sax2->locator, "Out of memory");
-	return;
+      if(!dst) {
+        raptor_log_error_to_handlers(sax2->error_handlers, 
+                                     RAPTOR_LOG_LEVEL_FATAL,
+                                     sax2->locator, "Out of memory");
+        return;
       }
 
       atts[i+1]=dst;
@@ -662,14 +790,14 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
       while (*src == 0x20 || *src == 0x0d || *src == 0x0a || *src == 0x09) 
         src++;
       while (*src) {
-	if (*src == 0x20 || *src == 0x0d || *src == 0x0a || *src == 0x09) {
+        if (*src == 0x20 || *src == 0x0d || *src == 0x0a || *src == 0x09) {
           while (*src == 0x20 || *src == 0x0d || *src == 0x0a || *src == 0x09)
             src++;
           if (*src)
             *dst++ = 0x20;
-	} else {
+        } else {
           *dst++ = *src++;
-	}
+        }
       }
       *dst = '\0';
       xmlFree(value);
@@ -685,10 +813,12 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
     /* Save passed in XML attributes pointers so we can 
      * NULL the pointers when they get handled below (various atts[i]=NULL)
      */
-    for (i = 0; atts[i]; i++);
+    for (i = 0; atts[i]; i++) ;
     xml_atts_size=sizeof(unsigned char*) * i;
     if(xml_atts_size) {
       xml_atts_copy=(unsigned char**)RAPTOR_MALLOC(cstringpointer,xml_atts_size);
+      if(!xml_atts_copy)
+        goto fail;
       memcpy(xml_atts_copy, atts, xml_atts_size);
     }
 
@@ -728,8 +858,10 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
       } else if(!strcmp((char*)atts[i], "xml:lang")) {
         xml_language=(unsigned char*)RAPTOR_MALLOC(cstring, strlen((char*)atts[i+1])+1);
         if(!xml_language) {
-          sax2->fatal_error_handler(sax2->fatal_error_data, sax2->locator, "Out of memory");
-          return;
+          raptor_log_error_to_handlers(sax2->error_handlers, 
+                                       RAPTOR_LOG_LEVEL_FATAL,
+                                       sax2->locator, "Out of memory");
+          goto fail;
         }
 
         /* optionally normalize language to lowercase */
@@ -765,13 +897,16 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
   el_name=raptor_new_qname(&sax2->namespaces, name, NULL,
                            (raptor_simple_message_handler)raptor_sax2_simple_error, sax2);
   if(!el_name)
-    return;
+    goto fail;
 
   xml_element=raptor_new_xml_element(el_name, xml_language, xml_base);
   if(!xml_element) {
     raptor_free_qname(el_name);
-    return;
+    goto fail;
   }
+  /* xml_language,xml_base now owned by xml_element */
+  xml_language = NULL;
+  xml_base = NULL; 
 
   /* Turn string attributes into namespaced-attributes */
   if(ns_attributes_count) {
@@ -783,10 +918,10 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
                                               ns_attributes_count, 
                                               sizeof(raptor_qname*));
     if(!named_attrs) {
-      sax2->fatal_error_handler(sax2->fatal_error_data, sax2->locator, "Out of memory");
-      RAPTOR_FREE(raptor_xml_element, xml_element);
-      raptor_free_qname(raptor_xml_element_get_name(xml_element));
-      return;
+      raptor_log_error_to_handlers(sax2->error_handlers, 
+                                   RAPTOR_LOG_LEVEL_FATAL,
+                                   sax2->locator, "Out of memory");
+      goto fail;
     }
 
     for (i = 0; i < all_atts_count; i++) {
@@ -806,8 +941,7 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
         for (j=0; j < i; j++)
           RAPTOR_FREE(raptor_qname, named_attrs[j]);
         RAPTOR_FREE(raptor_qname_array, named_attrs);
-        raptor_free_xml_element(xml_element);
-        return;
+        goto fail;
       }
 
       named_attrs[offset++]=attr;
@@ -830,6 +964,17 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
     RAPTOR_FREE(cstringpointer, xml_atts_copy);
   }
 
+  return;
+
+  fail:
+  if(xml_atts_copy)
+    RAPTOR_FREE(cstringpointer, xml_atts_copy);
+  if(xml_base)
+    raptor_free_uri(xml_base);
+  if(xml_language)
+    RAPTOR_FREE(cstring, xml_language);
+  if(xml_element)
+    raptor_free_xml_element(xml_element);
 }
 
 
@@ -935,5 +1080,13 @@ raptor_sax2_external_entity_ref(void* user_data,
   if(sax2->external_entity_ref_handler)
     return sax2->external_entity_ref_handler(sax2->user_data,
                                              context, base, systemId, publicId);
+
+  raptor_sax2_simple_error((void*)sax2,
+                           "Failed to handle external entity reference with base %s systemId %s publicId %s",
+                           (base ?  (const char*)base : "(None)"), 
+                           systemId,
+                           (publicId ?  (const char*)publicId: "(None)"));
+  
+  /* Failed to handle external entity reference */
   return 0;
 }
