@@ -77,6 +77,9 @@ typedef struct {
    */
   raptor_namespace* default_nspace;
 
+  /* the xml: namespace */
+  raptor_namespace *xml_nspace;
+
   /* the root element (rdf:RDF or atom:feed) */
   raptor_xml_element* root_element;
 
@@ -86,6 +89,8 @@ typedef struct {
   /* non-0 if this is an atom 1.0 serializer */
   int is_atom;
 
+  /* namespaces declared here */
+  raptor_namespace* nspaces[RAPTOR_RSS_NAMESPACES_SIZE];
 } raptor_rss10_serializer_context;
 
 
@@ -136,13 +141,16 @@ raptor_rss10_serialize_terminate(raptor_serializer* serializer)
     raptor_free_xml_writer(rss_serializer->xml_writer);
 
   for(i=0; i<RAPTOR_RSS_NAMESPACES_SIZE;i++) {
-    if(raptor_rss_namespaces_info[i].nspace)
-      raptor_free_namespace(raptor_rss_namespaces_info[i].nspace);
+    if(rss_serializer->nspaces[i])
+      raptor_free_namespace(rss_serializer->nspaces[i]);
   }
   
   if(rss_serializer->default_nspace)
     raptor_free_namespace(rss_serializer->default_nspace);
 
+  if(rss_serializer->xml_nspace)
+    raptor_free_namespace(rss_serializer->xml_nspace);
+
   if(rss_serializer->nstack)
     raptor_free_namespaces(rss_serializer->nstack);
 
@@ -386,12 +394,15 @@ raptor_rss10_serialize_statement(raptor_serializer* serializer,
                          RAPTOR_RSS_RDF_Seq_URI(rss_model))) {
 
       /* triple (?resource rdf:type rdf:Seq) */
-      RAPTOR_DEBUG2("Saw rdf:Seq with URI <%s>\n",
-                    raptor_uri_as_string((raptor_uri*)statement->subject));
-      if(statement->subject_type==RAPTOR_IDENTIFIER_TYPE_ANONYMOUS)
+      if(statement->subject_type==RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
+        RAPTOR_DEBUG2("Saw rdf:Seq with blank node %s\n",
+                      (char*)statement->subject);
         rss_serializer->seq_uri=raptor_new_uri((unsigned char*)statement->subject);
-      else
+      } else {
+        RAPTOR_DEBUG2("Saw rdf:Seq with URI <%s>\n",
+                      raptor_uri_as_string((raptor_uri*)statement->subject));
         rss_serializer->seq_uri=raptor_uri_copy(rss_serializer->seq_uri);
+      }
       
       handled=1;
     } else {
@@ -557,6 +568,11 @@ raptor_rss10_build_xml_names(raptor_serializer *serializer)
                                                       (is_atom ? raptor_atom_namespace_uri : raptor_rdf_namespace_uri),
                                                       0);
   
+  rss_serializer->xml_nspace=raptor_new_namespace(rss_serializer->nstack,
+                                                  (const unsigned char*)"xml",
+                                                  (const unsigned char*)raptor_xml_namespace_uri,
+                                                  0);
+
   qname=raptor_new_qname_from_namespace_local_name(rss_serializer->default_nspace, (is_atom ? (const unsigned char*)"feed" : (const unsigned char*)"RDF"),  NULL);
   if(base_uri)
     base_uri=raptor_uri_copy(base_uri);
@@ -577,7 +593,7 @@ raptor_rss10_build_xml_names(raptor_serializer *serializer)
       if(is_default_ns)
         prefix=NULL;
       nspace=raptor_new_namespace(rss_serializer->nstack, prefix, raptor_uri_as_string(uri), 0);
-      raptor_rss_namespaces_info[i].nspace=nspace;
+      rss_serializer->nspaces[i]=nspace;
       
       raptor_xml_element_declare_namespace(element, nspace);
     }
@@ -585,14 +601,14 @@ raptor_rss10_build_xml_names(raptor_serializer *serializer)
 
   for(i=0; i< RAPTOR_RSS_FIELDS_SIZE; i++) {
     int n=raptor_rss_fields_info[i].nspace;
-    raptor_namespace* nspace=raptor_rss_namespaces_info[n].nspace;
+    raptor_namespace* nspace=rss_serializer->nspaces[n];
     raptor_rss_fields_info[i].qname=raptor_new_qname_from_namespace_local_name(nspace,
                                                                                (const unsigned char*)raptor_rss_fields_info[i].name, NULL);
   }
 
   for(i=0; i< RAPTOR_RSS_COMMON_SIZE; i++) {
     int n=raptor_rss_types_info[i].nspace;
-    raptor_namespace* nspace=raptor_rss_namespaces_info[n].nspace;
+    raptor_namespace* nspace=rss_serializer->nspaces[n];
     if(nspace)
       raptor_rss_types_info[i].qname=raptor_new_qname_from_namespace_local_name(nspace, (const unsigned char*)raptor_rss_types_info[i].name, NULL);
   }
@@ -626,7 +642,7 @@ raptor_rss10_build_xml_names(raptor_serializer *serializer)
 }
 
 
-static const unsigned char *raptor_rss10_spaces=(const unsigned char*)"          ";
+static const unsigned char * const raptor_rss10_spaces=(const unsigned char*)"          ";
 
 static void
 raptor_rss10_emit_item(raptor_serializer* serializer,
@@ -764,11 +780,11 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
             attrs[attr_count]=raptor_new_qname_from_namespace_local_name(rss_serializer->default_nspace, (const unsigned char*)"resource",  raptor_uri_as_string(field->uri));
             attr_count++;
             if (enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_TYPE] && enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_TYPE]->value) {
-              attrs[attr_count]=raptor_new_qname_from_namespace_local_name(raptor_rss_namespaces_info[RSS2_0_ENC_NS].nspace, (const unsigned char*)raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE_TYPE].name, (const unsigned char*)enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_TYPE]->value);
+              attrs[attr_count]=raptor_new_qname_from_namespace_local_name(rss_serializer->nspaces[RSS2_0_ENC_NS], (const unsigned char*)raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE_TYPE].name, (const unsigned char*)enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_TYPE]->value);
               attr_count++;
             }
             if (enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_LENGTH] && enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_LENGTH]->value) {
-              attrs[attr_count]=raptor_new_qname_from_namespace_local_name(raptor_rss_namespaces_info[RSS2_0_ENC_NS].nspace, (const unsigned char*)raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE_LENGTH].name, (const unsigned char*)enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_LENGTH]->value);
+              attrs[attr_count]=raptor_new_qname_from_namespace_local_name(rss_serializer->nspaces[RSS2_0_ENC_NS], (const unsigned char*)raptor_rss_fields_info[RAPTOR_RSS_RDF_ENCLOSURE_LENGTH].name, (const unsigned char*)enclosure_item->fields[RAPTOR_RSS_RDF_ENCLOSURE_LENGTH]->value);
               attr_count++;
             }
             raptor_xml_element_set_attributes(predicate, attrs, attr_count);
@@ -876,12 +892,14 @@ raptor_rss10_serialize_end(raptor_serializer* serializer) {
   raptor_rss_model* rss_model;
   int i;
   raptor_xml_writer* xml_writer;
-  raptor_uri_handler *uri_handler;
+  const raptor_uri_handler *uri_handler;
   void *uri_context;
 #ifdef RAPTOR_DEBUG
   int triple_count=0;
 #endif
   int is_atom;
+  raptor_qname **attrs;
+  int attrs_count=0;
   
   rss_model=&rss_serializer->model;
   is_atom=rss_serializer->is_atom;
@@ -913,6 +931,9 @@ raptor_rss10_serialize_end(raptor_serializer* serializer) {
                                                NULL, NULL, /* errors */
                                                1);
 
+  if(rss_serializer->xml_writer)
+    raptor_free_xml_writer(rss_serializer->xml_writer);
+
   xml_writer=raptor_new_xml_writer(rss_serializer->nstack,
                                    uri_handler, uri_context,
 
@@ -923,6 +944,21 @@ raptor_rss10_serialize_end(raptor_serializer* serializer) {
 
   raptor_rss10_build_xml_names(serializer);
 
+  if(serializer->base_uri) {
+    const unsigned char* base_uri_string;
+
+    attrs=(raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, sizeof(raptor_qname*));
+
+    base_uri_string=raptor_uri_as_string(serializer->base_uri);
+    attrs[attrs_count++]=raptor_new_qname_from_namespace_local_name(rss_serializer->xml_nspace, (const unsigned char*)"base",  base_uri_string);
+  }
+
+  if(attrs_count)
+    raptor_xml_element_set_attributes(rss_serializer->root_element, attrs, 
+                                      attrs_count);
+  else
+    raptor_xml_element_set_attributes(rss_serializer->root_element, NULL, 0);
+
   raptor_xml_writer_start_element(xml_writer, rss_serializer->root_element);
 
   raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
@@ -980,7 +1016,7 @@ raptor_rss10_serialize_finish_factory(raptor_serializer_factory* factory)
 }
 
 
-static void
+static int
 raptor_rss10_serializer_register_factory(raptor_serializer_factory *factory)
 {
   factory->context_length     = sizeof(raptor_rss10_serializer_context);
@@ -992,25 +1028,27 @@ raptor_rss10_serializer_register_factory(raptor_serializer_factory *factory)
   factory->serialize_statement = raptor_rss10_serialize_statement;
   factory->serialize_end       = raptor_rss10_serialize_end;
   factory->finish_factory      = raptor_rss10_serialize_finish_factory;
+
+  return 0;
 }
 
 
 
-void
+int
 raptor_init_serializer_rss10(void) {
-  raptor_serializer_register_factory("rss-1.0",  "RSS 1.0",
-                                     NULL, 
-                                     NULL,
-                                     (const unsigned char*)"http://purl.org/rss/1.0/spec",
-                                     &raptor_rss10_serializer_register_factory);
+  return raptor_serializer_register_factory("rss-1.0",  "RSS 1.0",
+                                            NULL, 
+                                            NULL,
+                                            (const unsigned char*)"http://purl.org/rss/1.0/spec",
+                                            &raptor_rss10_serializer_register_factory);
 }
 
-void
+int
 raptor_init_serializer_atom(void) {
-  raptor_serializer_register_factory("atom",  "Atom 1.0",
-                                     "application/atom+xml", 
-                                     NULL,
-                                     NULL,
-                                     &raptor_rss10_serializer_register_factory);
+  return raptor_serializer_register_factory("atom",  "Atom 1.0",
+                                            "application/atom+xml", 
+                                            NULL,
+                                            NULL,
+                                            &raptor_rss10_serializer_register_factory);
 }
 
