@@ -3,7 +3,7 @@
  * raptor_turtle_writer.c - Raptor Turtle Writer
  *
  * Copyright (C) 2006, Dave Robillard
- * Copyright (C) 2003-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2008, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -46,6 +46,7 @@
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif
+#include <math.h>
 
 /* Raptor includes */
 #include "raptor.h"
@@ -72,7 +73,7 @@ struct raptor_turtle_writer_s {
   raptor_namespace_stack *nstack;
   int nstack_depth;
 
-  raptor_uri_handler *uri_handler;
+  const raptor_uri_handler *uri_handler;
   void *uri_context;
 
   raptor_simple_message_handler error_handler;
@@ -86,11 +87,22 @@ struct raptor_turtle_writer_s {
 
   /* indentation per level if formatting */
   int indent;
+
+  raptor_uri* xsd_boolean_uri;
+  raptor_uri* xsd_decimal_uri;
+  raptor_uri* xsd_double_uri;
+  raptor_uri* xsd_integer_uri;
 };
 
-#define SPACES_BUFFER_SIZE 16
-static unsigned char spaces_buffer[SPACES_BUFFER_SIZE];
-static int spaces_inited = 0;
+
+/* 16 spaces */
+#define SPACES_BUFFER_SIZE sizeof(spaces_buffer)
+static const unsigned char spaces_buffer[] = {
+  ' ', ' ', ' ', ' ',
+  ' ', ' ', ' ', ' ',
+  ' ', ' ', ' ', ' ',
+  ' ', ' ', ' ', ' '
+};
 
 
 void
@@ -112,14 +124,6 @@ raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer)
 {
   int num_spaces;
   
-  if(!spaces_inited) {
-    int i;
-    for(i = 0; i < SPACES_BUFFER_SIZE; i++)
-      spaces_buffer[i] = ' ';
-    
-    spaces_inited = 1;
-  }
-
   raptor_iostream_write_byte(turtle_writer->iostr, '\n');
  
   if(!TURTLE_WRITER_AUTO_INDENT(turtle_writer))
@@ -129,7 +133,7 @@ raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer)
 
   while(num_spaces > 0) {
     int count;
-    count = (num_spaces > SPACES_BUFFER_SIZE) ? SPACES_BUFFER_SIZE : num_spaces;
+    count = (num_spaces > (int)SPACES_BUFFER_SIZE) ? (int)SPACES_BUFFER_SIZE : num_spaces;
 
     raptor_iostream_write_counted_string(turtle_writer->iostr, spaces_buffer, count);
 
@@ -143,6 +147,7 @@ raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer)
 /**
  * raptor_new_turtle_writer:
  * @base_uri: Base URI for the writer
+ * @write_base_uri: non-0 to write 'base' in output
  * @nstack: Namespace stack for the writer to start with (or NULL)
  * @uri_handler: URI handler function
  * @uri_context: URI handler context data
@@ -155,9 +160,9 @@ raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer)
  * Return value: a new #raptor_turtle_writer object or NULL on failure
  **/
 raptor_turtle_writer*
-raptor_new_turtle_writer(raptor_uri* base_uri,
+raptor_new_turtle_writer(raptor_uri* base_uri, int write_base_uri,
                          raptor_namespace_stack *nstack,
-                         raptor_uri_handler *uri_handler,
+                         const raptor_uri_handler *uri_handler,
                          void *uri_context,
                          raptor_iostream* iostr,
                          raptor_simple_message_handler error_handler,
@@ -170,8 +175,6 @@ raptor_new_turtle_writer(raptor_uri* base_uri,
 
   turtle_writer->nstack_depth=0;
 
-  turtle_writer->base_uri = base_uri;
-
   turtle_writer->uri_handler=uri_handler;
   turtle_writer->uri_context=uri_context;
 
@@ -192,6 +195,14 @@ raptor_new_turtle_writer(raptor_uri* base_uri,
 
   turtle_writer->flags = 0;
   turtle_writer->indent = 2;
+
+  if(base_uri && write_base_uri)
+    raptor_turtle_writer_base(turtle_writer, base_uri);
+
+  turtle_writer->xsd_boolean_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#boolean");
+  turtle_writer->xsd_decimal_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
+  turtle_writer->xsd_double_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#double");
+  turtle_writer->xsd_integer_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
   
   return turtle_writer;
 }
@@ -210,6 +221,15 @@ raptor_free_turtle_writer(raptor_turtle_writer* turtle_writer)
   if(turtle_writer->nstack && turtle_writer->my_nstack)
     raptor_free_namespaces(turtle_writer->nstack);
 
+  if(turtle_writer->xsd_boolean_uri)
+    raptor_free_uri(turtle_writer->xsd_boolean_uri);
+  if(turtle_writer->xsd_decimal_uri)
+    raptor_free_uri(turtle_writer->xsd_decimal_uri);
+  if(turtle_writer->xsd_double_uri)
+    raptor_free_uri(turtle_writer->xsd_double_uri);
+  if(turtle_writer->xsd_integer_uri)
+    raptor_free_uri(turtle_writer->xsd_integer_uri);
+
   RAPTOR_FREE(raptor_turtle_writer, turtle_writer);
 }
 
@@ -277,10 +297,32 @@ raptor_turtle_writer_namespace_prefix(raptor_turtle_writer* turtle_writer,
   if(ns->prefix)
     raptor_iostream_write_string(turtle_writer->iostr, 
                                  raptor_namespace_get_prefix(ns));
-  raptor_iostream_write_string(turtle_writer->iostr, ": <");
-  raptor_iostream_write_string(turtle_writer->iostr,
-                               raptor_uri_as_string(raptor_namespace_get_uri(ns)));
-  raptor_iostream_write_string(turtle_writer->iostr, "> .\n");
+  raptor_iostream_write_counted_string(turtle_writer->iostr, ": ", 2);
+  raptor_turtle_writer_reference(turtle_writer, raptor_namespace_get_uri(ns));
+  raptor_iostream_write_counted_string(turtle_writer->iostr, " .\n", 3);
+}
+
+
+/**
+ * raptor_turtle_writer_base:
+ * @turtle_writer: Turtle writer object
+ * @base_uri: New base URI or NULL
+ *
+ * Write a base URI directive (@base) to set the in-scope base URI
+ */
+void
+raptor_turtle_writer_base(raptor_turtle_writer* turtle_writer,
+                          raptor_uri* base_uri)
+{
+  if(base_uri) {
+    raptor_iostream_write_counted_string(turtle_writer->iostr, "@base ", 6);
+    raptor_turtle_writer_reference(turtle_writer, base_uri);
+    raptor_iostream_write_counted_string(turtle_writer->iostr, " .\n", 3);
+  }
+  
+  if(turtle_writer->base_uri)
+    raptor_free_uri(turtle_writer->base_uri);
+  turtle_writer->base_uri=base_uri;
 }
 
 
@@ -302,7 +344,7 @@ raptor_turtle_writer_reference(raptor_turtle_writer* turtle_writer,
   uri_str = raptor_uri_to_relative_counted_uri_string(turtle_writer->base_uri, uri, &length);
 
   raptor_iostream_write_byte(turtle_writer->iostr, '<');
-  if (uri_str)
+  if(uri_str)
     raptor_iostream_write_string_ntriples(turtle_writer->iostr,
                                           uri_str, length, '>');
   raptor_iostream_write_byte(turtle_writer->iostr, '>');
@@ -336,127 +378,161 @@ raptor_turtle_writer_qname(raptor_turtle_writer* turtle_writer,
 }
 
 
+/**
+ * raptor_iostream_write_string_python:
+ * @iostr: #raptor_iostream to write to
+ * @string: UTF-8 string to write
+ * @len: length of UTF-8 string
+ * @delim: Terminating delimiter character for string (such as " or >)
+ * or \0 for no escaping.
+ * @flags: flags 0=N-Triples mode, 1=Turtle (allow raw UTF-8), 2=Turtle long string (allow raw UTF-8), 3=JSON
+ *
+ * Write a UTF-8 string using Python-style escapes (N-Triples, Turtle, JSON) to an iostream.
+ * 
+ * Return value: non-0 on failure such as bad UTF-8 encoding.
+ **/
+int
+raptor_iostream_write_string_python(raptor_iostream *iostr,
+                                    const unsigned char *string,
+                                    size_t len,
+                                    const char delim,
+                                    int flags)
+{
+  unsigned char c;
+  int unichar_len;
+  raptor_unichar unichar;
+
+  if(flags < 0 || flags > 3)
+    return 1;
+  
+  for(; (c=*string); string++, len--) {
+    if((delim && c == delim && (delim=='\'' || delim == '"')) ||
+       c == '\\') {
+      raptor_iostream_write_byte(iostr, '\\');
+      raptor_iostream_write_byte(iostr, c);
+      continue;
+    }
+    if(delim && c == delim) {
+      raptor_iostream_write_counted_string(iostr, "\\u", 2);
+      raptor_iostream_format_hexadecimal(iostr, c, 4);
+      continue;
+    }
+    
+    if(flags != 2) {
+      /* N-Triples, Turtle or JSON */
+
+      /* Note: NTriples is ASCII */
+      if(c == 0x09) {
+        raptor_iostream_write_counted_string(iostr, "\\t", 2);
+        continue;
+      } else if((flags == 3) && c == 0x08) {
+        /* JSON has \b for backspace */
+        raptor_iostream_write_counted_string(iostr, "\\b", 2);
+        continue;
+      } else if(c == 0x0a) {
+        raptor_iostream_write_counted_string(iostr, "\\n", 2);
+        continue;
+      } else if((flags == 3) && c == 0x0b) {
+        /* JSON has \f for formfeed */
+        raptor_iostream_write_counted_string(iostr, "\\f", 2);
+        continue;
+      } else if(c == 0x0d) {
+        raptor_iostream_write_counted_string(iostr, "\\r", 2);
+        continue;
+      } else if(c < 0x20|| c == 0x7f) {
+        raptor_iostream_write_counted_string(iostr, "\\u", 2);
+        raptor_iostream_format_hexadecimal(iostr, c, 4);
+        continue;
+      } else if(c < 0x80) {
+        raptor_iostream_write_byte(iostr, c);
+        continue;
+      }
+    } else if(c < 0x80) {
+      /* Turtle long string has no escapes except delim */
+      raptor_iostream_write_byte(iostr, c);
+      continue;
+    } 
+    
+    /* It is unicode */
+    
+    unichar_len=raptor_utf8_to_unicode_char(NULL, string, len);
+    if(unichar_len < 0 || unichar_len > (int)len)
+      /* UTF-8 encoding had an error or ended in the middle of a string */
+      return 1;
+
+    if(flags >= 1 && flags <= 3) {
+      /* Turtle and JSON are UTF-8 - no need to escape */
+      raptor_iostream_write_counted_string(iostr, string, unichar_len);
+    } else {
+      unichar_len=raptor_utf8_to_unicode_char(&unichar, string, len);
+
+      if(unichar < 0x10000) {
+        raptor_iostream_write_counted_string(iostr, "\\u", 2);
+        raptor_iostream_format_hexadecimal(iostr, unichar, 4);
+      } else {
+        raptor_iostream_write_counted_string(iostr, "\\U", 2);
+        raptor_iostream_format_hexadecimal(iostr, unichar, 8);
+      }
+    }
+    
+    unichar_len--; /* since loop does len-- */
+    string += unichar_len; len -= unichar_len;
+
+  }
+
+  return 0;
+}
+
+
 /**
  * raptor_iostream_write_string_turtle:
  * @iostr: #raptor_iostream to write to
  * @string: UTF-8 string to write
  * @len: length of UTF-8 string
  *
- * Write an UTF-8 string using Turtle "longString" triple quoting to an iostream.
+ * Write an UTF-8 string using Turtle "longString" triple quoting to
+ * an iostream.
+ *
  **/
 void
 raptor_iostream_write_string_turtle(raptor_iostream *iostr,
-                                    const unsigned char *string,
-                                    size_t len)
+                                    const unsigned char *string, size_t len)
 {
-  unsigned char c;
-  
-  for(; (c=*string); string++, len--) {
-    if(c == '\"') {
-      raptor_iostream_write_byte(iostr, '\\');
-      raptor_iostream_write_byte(iostr, c);
-    } else if(c == '\\') {
-      raptor_iostream_write_byte(iostr, '\\');
-      raptor_iostream_write_byte(iostr, '\\');
-    } else {
-      raptor_iostream_write_byte(iostr, c);
-    }
-  }
+  raptor_iostream_write_string_python(iostr, string, len, '"', 1);
 }
 
 
+
 /**
- * raptor_turtle_writer_quoted:
+ * raptor_turtle_writer_quoted_counted_string:
  * @turtle_writer: Turtle writer object
- * @s: string to write (SHARED)
+ * @s: string to write
+ * @len: string length
  *
- * Write a string raw to the Turtle writer.
+ * Write a Turtle escaped-string inside double quotes to the writer.
+ *
+ * Return value: non-0 on failure
  **/
-void
-raptor_turtle_writer_quoted(raptor_turtle_writer* turtle_writer,
-                            unsigned char *s)
+int
+raptor_turtle_writer_quoted_counted_string(raptor_turtle_writer* turtle_writer,
+                                           const unsigned char *s, size_t len)
 {
-  raptor_stringbuffer* sb = raptor_new_stringbuffer();
-
-  if(raptor_turtle_writer_contains_newline(s)) {
-    raptor_iostream_write_string(turtle_writer->iostr, "\"\"\"");
-    raptor_iostream_write_string_turtle(turtle_writer->iostr, s, 
-                                        strlen((const char*)s));
-    raptor_iostream_write_string(turtle_writer->iostr, "\"\"\"");
-  } else {
-    raptor_stringbuffer_append_turtle_string(sb, s, strlen((const char*)s),
-                                             (int)'"',
-        turtle_writer->error_handler, turtle_writer->error_data);
-
-    raptor_iostream_write_byte(turtle_writer->iostr, '\"');
-
-    /* FIXME: over-escapes things because ntriples is ASCII */
-    raptor_iostream_write_string_ntriples(turtle_writer->iostr, s,
-                                          strlen((const char*)s), '"');
-    raptor_iostream_write_byte(turtle_writer->iostr, '\"');
-  }
-
-  raptor_free_stringbuffer(sb);
-}
-
-
-/* Convert num into turtle xsd:double format in buf.
- * Output form is xsd:double canonical form, excluding NaN, INF, -INF.
- * Returns nonzero on failure (num is unrepresentable) */
-static int
-snprint_turtle_double(char* buf, size_t buf_len, double num)
-{
-  size_t e_index = 0;
-  size_t trailing_zero_start = 0;
-  size_t exponent_start;
+  const unsigned char *quotes=(const unsigned char *)"\"\"\"\"";
+  const unsigned char *q;
+  size_t q_len;
+  int flags;
   
-  if(num == 0.0f) {
-    strcpy(buf, "0.0E0");
-    return 0;
-  }
-
-  snprintf(buf, buf_len, "%1.14E", num);
-
-  /* unrepresentable in turtle */
-  if(!strcmp(buf, "NAN") || !strcmp(buf, "INF") || !strcmp(buf, "-INF")) {
-    strcpy(buf, "");
-    return -1;
-  }
+  if(!s)
+    return 1;
   
-  /* now munge snprintf output into turtle form (yes, ick) */
-
-  /* find the 'E' and start of mantissa trailing zeros */
-
-  for( ; e_index < strlen(buf); ++e_index) {
-    if(e_index > 0 && buf[e_index] == '0' && buf[e_index-1] != '0')
-      trailing_zero_start = e_index;
-    else if(buf[e_index] == 'E')
-      break;
-  }
-
-  if(buf[trailing_zero_start-1] == '.')
-    ++trailing_zero_start;
-
-  /* write an 'E' where the trailing zeros started */
-  buf[trailing_zero_start] = 'E';
-  if(buf[e_index+1] == '-') {
-    buf[trailing_zero_start+1] = '-';
-    ++trailing_zero_start;
-  }
-
-  exponent_start = e_index+2;
-
-  while(exponent_start < strlen(buf) && buf[exponent_start] == '0')
-    ++exponent_start;
-
-  if(exponent_start == strlen(buf)) {
-    buf[trailing_zero_start+1] = '0';
-    buf[trailing_zero_start+2] = '\0';
-  } else {
-    /* copy the exponent (minus leading zeros) after the new E */
-    memmove(buf+trailing_zero_start+1, buf+exponent_start,
-            strlen(buf)-trailing_zero_start);
-  }
+  /* Turtle """longstring""" (2) or "string" (1) */
+  flags=raptor_turtle_writer_contains_newline(s) ? 2 : 1;
+  q=(flags == 2) ? quotes : quotes+2;
+  q_len=(q == quotes) ? 3 : 1;
+  raptor_iostream_write_counted_string(turtle_writer->iostr, q, q_len);
+  raptor_iostream_write_string_python(turtle_writer->iostr,
+                                      s, strlen((const char*)s), '"', flags);
+  raptor_iostream_write_counted_string(turtle_writer->iostr, q, q_len);
 
   return 0;
 }
@@ -465,57 +541,39 @@ snprint_turtle_double(char* buf, size_t buf_len, double num)
 /**
  * raptor_turtle_writer_literal:
  * @turtle_writer: Turtle writer object
+ * @nstack: Namespace stack for making a QName for datatype URI
  * @s: literal string to write (SHARED)
  * @lang: language tag (may be NULL)
  * @datatype: datatype URI (may be NULL)
  *
  * Write a literal (possibly with lang and datatype) to the Turtle writer.
+ *
+ * Return value: non-0 on failure
  **/
-void
+int
 raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer,
-                             unsigned char* s, unsigned char* lang,
+                             raptor_namespace_stack *nstack,
+                             const unsigned char* s, const unsigned char* lang,
                              raptor_uri* datatype)
 {
-  char buf[40]; /* 20 is enough for 64 bit integer, I think 40 covers double.. */
-  char* endptr = (char *)s;
+  /* DBL_MAX = 309 decimal digits */
+  #define INT_MAX_LEN 309 
+
+  /* DBL_EPSILON = 52 digits */
+  #define FRAC_MAX_LEN 52
+
   int written = 0;
 
   /* typed literal special cases */
   if(datatype) {
-    const char* type_uri_str = (const char*)raptor_uri_as_string(datatype);
-
-    /* integer */
-    if(!strcmp(type_uri_str, "http://www.w3.org/2001/XMLSchema#integer")) {
-      long inum = strtol((const char*)s, NULL, 10);
-      if(inum != LONG_MIN && inum != LONG_MAX) {
-        snprintf(buf, 20, "%ld", inum);
-        raptor_iostream_write_string(turtle_writer->iostr, buf);
-        written = 1;
-      }
-    
-    /* double */
-    } else if(!strcmp(type_uri_str, "http://www.w3.org/2001/XMLSchema#double")) {
-      double dnum = strtod((const char*)s, &endptr);
-      if(endptr != (char*)s) {
-        if(!snprint_turtle_double(buf, 40, dnum)) {
-          raptor_iostream_write_string(turtle_writer->iostr, buf);
-          written = 1;
-        } else {
-          turtle_writer->error_handler(turtle_writer->error_data, "Illegal value for xsd:double literal.");
-        }
-      }
-    
-    /* decimal */
-    } else if(!strcmp(type_uri_str, "http://www.w3.org/2001/XMLSchema#decimal")) {
-      double dnum = strtod((const char*)s, &endptr);
-      if(endptr != (char*)s) {
-        snprintf(buf, 20, "%.1lf", dnum);
-        raptor_iostream_write_string(turtle_writer->iostr, buf);
-        written = 1;
-      }
-    
+    /* integer, double, decimal */
+    if(raptor_uri_equals(datatype, turtle_writer->xsd_integer_uri) ||
+       raptor_uri_equals(datatype, turtle_writer->xsd_double_uri) ||
+       raptor_uri_equals(datatype, turtle_writer->xsd_decimal_uri)) {
+      raptor_iostream_write_string(turtle_writer->iostr, s);
+      written = 1;
     /* boolean */
-    } else if(!strcmp(type_uri_str, "http://www.w3.org/2001/XMLSchema#boolean")) {
+    } else if(raptor_uri_equals(datatype, turtle_writer->xsd_boolean_uri)) {
       if(!strcmp((const char*)s, "0") || !strcmp((const char*)s, "false")) {
         raptor_iostream_write_string(turtle_writer->iostr, "false");
         written = 1;
@@ -529,20 +587,30 @@ raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer,
   }
 
   if(written)
-    return;
+    return 0;
     
-  raptor_turtle_writer_quoted(turtle_writer, s);
+  if(raptor_turtle_writer_quoted_counted_string(turtle_writer, s,
+                                                strlen((const char*)s)))
+    return 1;
 
   /* typed literal, not a special case */
   if(datatype) {
+    raptor_qname* qname;
+
     raptor_iostream_write_string(turtle_writer->iostr, "^^");
-    raptor_turtle_writer_reference(turtle_writer, datatype);
-  
-  /* literal with language tag */
+    qname = raptor_namespaces_qname_from_uri(nstack, datatype, 10);
+    if(qname) {
+      raptor_turtle_writer_qname(turtle_writer, qname);
+      raptor_free_qname(qname);
+    } else
+      raptor_turtle_writer_reference(turtle_writer, datatype);
   } else if(lang) {
-      raptor_iostream_write_byte(turtle_writer->iostr, '@');
-      raptor_iostream_write_string(turtle_writer->iostr, lang);
+    /* literal with language tag */
+    raptor_iostream_write_byte(turtle_writer->iostr, '@');
+    raptor_iostream_write_string(turtle_writer->iostr, lang);
   }
+
+  return 0;
 }
 
 
@@ -634,7 +702,7 @@ raptor_turtle_writer_set_feature(raptor_turtle_writer *turtle_writer,
       break;
     
     case RAPTOR_FEATURE_WRITER_AUTO_EMPTY:
-	case RAPTOR_FEATURE_WRITER_XML_VERSION:
+    case RAPTOR_FEATURE_WRITER_XML_VERSION:
     case RAPTOR_FEATURE_WRITER_XML_DECLARATION:
       break;
         
@@ -649,6 +717,10 @@ raptor_turtle_writer_set_feature(raptor_turtle_writer *turtle_writer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
@@ -665,6 +737,17 @@ raptor_turtle_writer_set_feature(raptor_turtle_writer *turtle_writer,
     case RAPTOR_FEATURE_LITERAL_FILL:
     case RAPTOR_FEATURE_BNODE_FILL:
 
+    /* JSON serializer features */
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    
+    /* Turtle serializer feature */
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
+
+    /* WWW feature */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+      
     default:
       return -1;
       break;
@@ -745,6 +828,10 @@ raptor_turtle_writer_get_feature(raptor_turtle_writer *turtle_writer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
@@ -761,6 +848,17 @@ raptor_turtle_writer_get_feature(raptor_turtle_writer *turtle_writer,
     case RAPTOR_FEATURE_LITERAL_FILL:
     case RAPTOR_FEATURE_BNODE_FILL:
 
+    /* JSON serializer features */
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    
+    /* Turtle serializer feature */
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
+
+    /* WWW feature */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+      
     default:
       break;
   }
@@ -799,26 +897,25 @@ int main(int argc, char *argv[]);
 
 const unsigned char *base_uri_string=(const unsigned char*)"http://example.org/base#";
 
-#define OUT_BYTES_COUNT 135
+const unsigned char* longstr=(const unsigned char*)"it's quoted\nand has newlines, \"s <> and\n\ttabbing";
+
+#define OUT_BYTES_COUNT 149
 
 int
 main(int argc, char *argv[]) 
 {
-#if 0
   const char *program=raptor_basename(argv[0]);
-  raptor_uri_handler *uri_handler;
+  const raptor_uri_handler *uri_handler;
   void *uri_context;
   raptor_iostream *iostr;
   raptor_namespace_stack *nstack;
-  raptor_namespace* foo_ns;
+  raptor_namespace* ex_ns;
   raptor_turtle_writer* turtle_writer;
   raptor_uri* base_uri;
   raptor_qname* el_name;
-  raptor_xml_element *element;
-  size_t count;
-  raptor_qname **attrs;
-  raptor_uri* base_uri_copy=NULL;
-
+  unsigned long count;
+  raptor_uri* datatype;
+  
   /* for raptor_new_iostream_to_string */
   void *string=NULL;
   size_t string_len=0;
@@ -837,76 +934,78 @@ main(int argc, char *argv[])
                                NULL, NULL, /* errors */
                                1);
 
-  turtle_writer=raptor_new_turtle_writer(nstack,
-                                   uri_handler, uri_context,
-                                   iostr,
-                                   NULL, NULL, /* errors */
-                                   1);
+  base_uri=raptor_new_uri(base_uri_string);
+
+  turtle_writer=raptor_new_turtle_writer(base_uri, 1,
+                                         nstack,
+                                         uri_handler, uri_context,
+                                         iostr,
+                                         NULL, NULL /* errors */
+                                         );
   if(!turtle_writer) {
     fprintf(stderr, "%s: Failed to create turtle_writer to iostream\n", program);
     exit(1);
   }
 
-  base_uri=raptor_new_uri(base_uri_string);
+  raptor_turtle_writer_set_feature(turtle_writer, 
+                                   RAPTOR_FEATURE_WRITER_AUTO_INDENT, 1);
 
-  foo_ns=raptor_new_namespace(nstack,
-                              (const unsigned char*)"foo",
-                              (const unsigned char*)"http://example.org/foo-ns#",
+  ex_ns=raptor_new_namespace(nstack,
+                              (const unsigned char*)"ex",
+                              (const unsigned char*)"http://example.org/ns#",
                               0);
 
 
-  el_name=raptor_new_qname_from_namespace_local_name(foo_ns,
+  raptor_turtle_writer_namespace_prefix(turtle_writer, ex_ns);
+
+  raptor_turtle_writer_reference(turtle_writer, base_uri);
+  
+  raptor_turtle_writer_increase_indent(turtle_writer);
+  raptor_turtle_writer_newline(turtle_writer);
+  
+  raptor_turtle_writer_raw(turtle_writer, (const unsigned char*)"ex:foo ");
+
+  raptor_turtle_writer_quoted_counted_string(turtle_writer, longstr,
+                                             strlen((const char*)longstr));
+  raptor_turtle_writer_raw_counted(turtle_writer,
+                                   (const unsigned char*)" ;", 2);
+  raptor_turtle_writer_newline(turtle_writer);
+
+  el_name=raptor_new_qname_from_namespace_local_name(ex_ns,
                                                      (const unsigned char*)"bar", 
                                                      NULL);
-  base_uri_copy=base_uri ? raptor_uri_copy(base_uri) : NULL;
-  element=raptor_new_xml_element(el_name,
-                                  NULL, /* language */
-                                  base_uri_copy);
 
-  raptor_turtle_writer_start_element(turtle_writer, element);
-  raptor_turtle_writer_cdata_counted(turtle_writer, (const unsigned char*)"hello\n", 6);
-  raptor_turtle_writer_comment_counted(turtle_writer, (const unsigned char*)"comment", 7);
-  raptor_turtle_writer_cdata(turtle_writer, (const unsigned char*)"\n");
-  raptor_turtle_writer_end_element(turtle_writer, element);
+  raptor_turtle_writer_qname(turtle_writer, el_name);
+  raptor_free_qname(el_name);
 
-  raptor_free_xml_element(element);
+  raptor_turtle_writer_raw_counted(turtle_writer, (const unsigned char*)" ", 1);
 
-  raptor_turtle_writer_cdata(turtle_writer, (const unsigned char*)"\n");
+  datatype=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
+  raptor_turtle_writer_literal(turtle_writer, nstack,
+                               (const unsigned char*)"10.0", NULL,
+                               datatype);
+  raptor_free_uri(datatype);
 
-  el_name=raptor_new_qname(nstack, 
-                           (const unsigned char*)"blah", 
-                           NULL, /* no attribute value - element */
-                           NULL, NULL); /* errors */
-  base_uri_copy=base_uri ? raptor_uri_copy(base_uri) : NULL;
-  element=raptor_new_xml_element(el_name,
-                                  NULL, /* language */
-                                  base_uri_copy);
+  raptor_turtle_writer_newline(turtle_writer);
 
-  attrs=(raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, sizeof(raptor_qname*));
-  attrs[0]=raptor_new_qname(nstack, 
-                            (const unsigned char*)"a",
-                            (const unsigned char*)"b", /* attribute value */
-                            NULL, NULL); /* errors */
-  raptor_xml_element_set_attributes(element, attrs, 1);
+  raptor_turtle_writer_decrease_indent(turtle_writer);
 
-  raptor_turtle_writer_empty_element(turtle_writer, element);
-
-  raptor_turtle_writer_cdata(turtle_writer, (const unsigned char*)"\n");
+  raptor_turtle_writer_raw_counted(turtle_writer, (const unsigned char*)".", 1);
+  raptor_turtle_writer_newline(turtle_writer);
 
+  
   raptor_free_turtle_writer(turtle_writer);
 
-  raptor_free_xml_element(element);
-
-  raptor_free_namespace(foo_ns);
+  raptor_free_namespace(ex_ns);
 
   raptor_free_namespaces(nstack);
 
   raptor_free_uri(base_uri);
 
   
-  count=raptor_iostream_get_bytes_written_count(iostr);
+  count=raptor_iostream_tell(iostr);
 
-#ifdef RAPTOR_DEBUG
+#if RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Freeing iostream\n", program);
 #endif
   raptor_free_iostream(iostr);
@@ -924,22 +1023,24 @@ main(int argc, char *argv[])
     fprintf(stderr, "%s: I/O stream failed to create a string\n", program);
     return 1;
   }
-  string_len=strlen(string);
+  string_len=strlen((const char*)string);
   if(string_len != count) {
     fprintf(stderr, "%s: I/O stream created a string length %d, expected %d\n", program, (int)string_len, (int)count);
     return 1;
   }
 
-  fprintf(stderr, "%s: Made XML string of %d bytes\n", program, (int)string_len);
+#if RAPTOR_DEBUG > 1
+  fprintf(stderr, "%s: Made Turtle string of %d bytes\n", program, (int)string_len);
   fputs("[[", stderr);
   (void)fwrite(string, 1, string_len, stderr);
   fputs("]]\n", stderr);
+#endif
 
   raptor_free_memory(string);
   
 
   raptor_finish();
-#endif
+
   /* keep gcc -Wall happy */
   return(0);
 }
