@@ -2,11 +2,8 @@
  *
  * raptor_utf8.c - Raptor UTF-8 and Unicode support
  *
- * $Id: raptor_utf8.c,v 1.22 2005/09/06 20:50:40 dajobe Exp $
- *
- * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -48,15 +45,21 @@
 #endif
 
 
-/*
- * Based on librdf_unicode_char_to_utf8
- * with no need to calculate length since the encoded character is
- * always copied into a buffer with sufficient size.
+/**
+ * raptor_unicode_char_to_utf8:
+ * @c: Unicode character
+ * @output: UTF-8 string buffer or NULL
+ *
+ * Convert a Unicode character to UTF-8 encoding.
+ * 
+ * Based on librdf_unicode_char_to_utf8() with no need to calculate
+ * length since the encoded character is always copied into a buffer
+ * with sufficient size.
  * 
  * Return value: bytes encoded to output buffer or <0 on failure
  **/
 int
-raptor_unicode_char_to_utf8(unsigned long c, unsigned char *output)
+raptor_unicode_char_to_utf8(raptor_unichar c, unsigned char *output)
 {
   int size=0;
   
@@ -113,20 +116,27 @@ raptor_unicode_char_to_utf8(unsigned long c, unsigned char *output)
   return size;
 }
 
-/*
- * Based on librdf_utf8_to_unicode_char
- * replacing librdf_unicode_char with long
- *
- * Return value: number of bytes used or <0 on failure
- */
 
+/**
+ * raptor_utf8_to_unicode_char:
+ * @output: Pointer to the Unicode character or NULL
+ * @input: UTF-8 string buffer
+ * @length: buffer size
+ *
+ * Convert an UTF-8 encoded buffer to a Unicode character.
+ * 
+ * If output is NULL, then will calculate the number of bytes that
+ * will be used from the input buffer and not perform the conversion.
+ * 
+ * Return value: bytes used from input buffer or <0 on failure: -1 input buffer too short or length error, -2 overlong UTF-8 sequence, -3 illegal code positions, -4 code out of range U+0000 to U+10FFFF.  In cases -2, -3 and -4 the coded character is stored in the output.
+ */
 int
-raptor_utf8_to_unicode_char(unsigned long *output,
+raptor_utf8_to_unicode_char(raptor_unichar *output,
                             const unsigned char *input, int length)
 {
   unsigned char in;
   int size;
-  unsigned long c=0;
+  raptor_unichar c=0;
   
   if(length < 1)
     return -1;
@@ -187,8 +197,42 @@ raptor_utf8_to_unicode_char(unsigned long *output,
       c |= in;
       /* FALLTHROUGH */
     default:
-      *output=c;
+      break;
   }
+  
+  *output=c;
+
+  /* check for overlong UTF-8 sequences */
+  switch(size) {
+    case 2:
+      if(c < 0x00000080)
+        return -2;
+      break;
+    case 3:
+      if(c < 0x00000800)
+        return -2;
+      break;
+    case 4:
+      if(c < 0x00010000)
+        return -2;
+      break;
+
+    default: /* 1 */
+      break;
+  }
+
+
+  /* check for illegal code positions:
+   * U+D800 to U+DFFF (UTF-16 surrogates)
+   * U+FFFE and U+FFFF
+   */
+  if((c > 0xD7FF && c < 0xE000) || c == 0xFFFE || c == 0xFFFF)
+    return -3;
+
+  /* Unicode 3.2 only defines U+0000 to U+10FFFF and UTF-8 encodings of it */
+  /* of course this makes some 4 byte forms illegal */
+  if(c > 0x10ffff)
+    return -4;
 
   return size;
 }
@@ -202,15 +246,24 @@ static int raptor_unicode_is_digit(long c);
 static int raptor_unicode_is_extender(long c);
 
 
+/**
+ * raptor_unicode_is_xml11_namestartchar:
+ * @c: Unicode character to check
+ * 
+ * Check if Unicode character is legal to start an XML 1.1 Name
+ * 
+ * Namespaces in XML 1.1 REC 2004-02-04
+ *   http://www.w3.org/TR/2004/REC-xml11-20040204/#NT-NameStartChar
+ * updating
+ *   Extensible Markup Language (XML) 1.1 REC 2004-02-04
+ *   http://www.w3.org/TR/2004/REC-xml11-20040204/ sec 2.3, [4a]
+ * excluding the ':'
+ *
+ * Return value: non-0 if legal
+ **/
 int
-raptor_unicode_is_xml11_namestartchar(long c) {
-  /* Namespaces in XML 1.1 REC 2004-02-04
-   *   http://www.w3.org/TR/2004/REC-xml11-20040204/#NT-NameStartChar
-   * updating
-   *   Extensible Markup Language (XML) 1.1 REC 2004-02-04
-   *   http://www.w3.org/TR/2004/REC-xml11-20040204/ sec 2.3, [4a]
-   * excluding the ':'
-   */
+raptor_unicode_is_xml11_namestartchar(raptor_unichar c)
+{
   return (((c >= 0x0041)  && (c <= 0x005A)) || /* [A-Z] */
           (c == 0x005F) ||                     /* '_' */
           ((c >= 0x0061)  && (c <= 0x007A)) || /* [a-z] */
@@ -229,15 +282,24 @@ raptor_unicode_is_xml11_namestartchar(long c) {
 }
 
 
+/**
+ * raptor_unicode_is_xml10_namestartchar:
+ * @c: Unicode character to check
+ *
+ * Check if Unicode character is legal to start an XML 1.0 Name
+ * 
+ * Namespaces in XML REC 1999-01-14
+ *   http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCName
+ * updating
+ *   Extensible Markup Language (XML) 1.0 (Third Edition) REC 2004-02-04
+ *   http://www.w3.org/TR/2004/REC-xml-20040204/
+ * excluding the ':'
+ *
+ * Return value: non-0 if legal
+ **/
 int
-raptor_unicode_is_xml10_namestartchar(long c) {
-  /* Namespaces in XML REC 1999-01-14
-   *   http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCName
-   * updating
-   *   Extensible Markup Language (XML) 1.0 (Third Edition) REC 2004-02-04
-   *   http://www.w3.org/TR/2004/REC-xml-20040204/
-   * excluding the ':'
-   */
+raptor_unicode_is_xml10_namestartchar(raptor_unichar c)
+{
   return (raptor_unicode_is_letter(c) ||
           (c == '_'));
 }
@@ -247,12 +309,12 @@ raptor_unicode_is_xml10_namestartchar(long c) {
  * raptor_unicode_is_namestartchar:
  * @c: Unicode character to check
  *
- * Check if Unicode character is legal to start an XML Namespace Name .
+ * Check if Unicode character is legal to start an XML Name
  * 
  * Return value: non-0 if the character is legal
  **/
 int
-raptor_unicode_is_namestartchar(long c) {
+raptor_unicode_is_namestartchar(raptor_unichar c) {
 #ifdef RAPTOR_XML_1_1
    return raptor_unicode_is_xml11_namestartchar(c);
 #else
@@ -261,15 +323,24 @@ raptor_unicode_is_namestartchar(long c) {
 }
 
 
+/**
+ * raptor_unicode_is_xml11_namechar:
+ * @c: Unicode character
+ * 
+ * Check if a Unicode codepoint is a legal to continue an XML 1.1 Name
+ *
+ * Namespaces in XML 1.1 REC 2004-02-04
+ *   http://www.w3.org/TR/2004/REC-xml11-20040204/
+ * updating
+ *   Extensible Markup Language (XML) 1.1 REC 2004-02-04
+ *   http://www.w3.org/TR/2004/REC-xml11-20040204/ sec 2.3, [4a]
+ * excluding the ':'
+ * 
+ * Return value: non-0 if legal
+ **/
 int
-raptor_unicode_is_xml11_namechar(long c) {
-  /* Namespaces in XML 1.1 REC 2004-02-04
-   *   http://www.w3.org/TR/2004/REC-xml11-20040204/
-   * updating
-   *   Extensible Markup Language (XML) 1.1 REC 2004-02-04
-   *   http://www.w3.org/TR/2004/REC-xml11-20040204/ sec 2.3, [4a]
-   * excluding the ':'
-   */
+raptor_unicode_is_xml11_namechar(raptor_unichar c)
+{
   return (raptor_unicode_is_xml11_namestartchar(c) ||
           (c == 0x002D) || /* '-' */
           (c == 0x002E) || /* '.' */
@@ -280,15 +351,24 @@ raptor_unicode_is_xml11_namechar(long c) {
 }
 
 
+/**
+ * raptor_unicode_is_xml10_namechar:
+ * @c: Unicode character
+ * 
+ * Check if a Unicode codepoint is a legal to continue an XML 1.0 Name
+ * 
+ * Namespaces in XML REC 1999-01-14
+ *   http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCNameChar
+ * updating
+ *   Extensible Markup Language (XML) 1.0 (Third Edition) REC 2004-02-04
+ *   http://www.w3.org/TR/2004/REC-xml-20040204/
+ * excluding the ':'
+ *
+ * Return value: non-0 if legal
+ **/
 int
-raptor_unicode_is_xml10_namechar(long c) {
-  /* Namespaces in XML REC 1999-01-14
-   *   http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCNameChar
-   * updating
-   *   Extensible Markup Language (XML) 1.0 (Third Edition) REC 2004-02-04
-   *   http://www.w3.org/TR/2004/REC-xml-20040204/
-   * excluding the ':'
-   */
+raptor_unicode_is_xml10_namechar(raptor_unichar c)
+{
   return (raptor_unicode_is_letter(c) ||
           raptor_unicode_is_digit(c) ||
           (c == 0x002E) || /* '.' */
@@ -303,12 +383,12 @@ raptor_unicode_is_xml10_namechar(long c) {
  * raptor_unicode_is_namechar:
  * @c: Unicode character to check
  *
- * Check if Unicode character is legal to continue an XML Namespace Name .
+ * Check if Unicode character is legal to continue an XML Name .
  * 
  * Return value: non-0 if the character is legal
  **/
 int
-raptor_unicode_is_namechar(long c) 
+raptor_unicode_is_namechar(raptor_unichar c) 
 {
 #ifdef RAPTOR_XML_1_1
    return raptor_unicode_is_xml11_namechar(c);
@@ -738,7 +818,7 @@ int
 raptor_utf8_check(const unsigned char *string, size_t length)
 {
   while(length > 0) {
-    unsigned long unichar=0;
+    raptor_unichar unichar=0;
 
     int unichar_len=raptor_utf8_to_unicode_char(&unichar, string, length);
     if(unichar_len < 0 || unichar_len > (int)length)
