@@ -2,7 +2,7 @@
  *
  * raptor_www_curl.c - Raptor WWW retrieval via libcurl
  *
- * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2008, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -42,6 +42,27 @@
 #include "raptor_internal.h"
 
 
+static void
+raptor_www_curl_update_status(raptor_www* www) 
+{
+  char* final_uri;
+    
+  if(www->failed)
+    return;
+  
+  if(www->checked_status++)
+    return;
+  
+  if(curl_easy_getinfo(www->curl_handle, CURLINFO_EFFECTIVE_URL, 
+                       &final_uri) == CURLE_OK) {
+    www->final_uri=raptor_new_uri((const unsigned char*)final_uri);
+    if(www->final_uri_handler)
+      www->final_uri_handler(www, www->final_uri_userdata, www->final_uri);
+  }
+
+}
+
+
 static size_t
 raptor_www_curl_write_callback(void *ptr, size_t size, size_t nmemb, void *userdata) 
 {
@@ -54,6 +75,8 @@ raptor_www_curl_write_callback(void *ptr, size_t size, size_t nmemb, void *userd
   if(www->failed)
     return 0;
   
+  raptor_www_curl_update_status(www);
+
 #if RAPTOR_DEBUG > 2
   RAPTOR_DEBUG2("Got %d bytes\n", bytes);
 #endif
@@ -83,7 +106,10 @@ raptor_www_curl_header_callback(void* ptr,  size_t  size, size_t nmemb,
     char *type_buffer=(char*)RAPTOR_MALLOC(cstring, len+1);
     strncpy(type_buffer, (char*)ptr+14, len);
     type_buffer[len]='\0';
+    if(www->type)
+      RAPTOR_FREE(cstring, www->type);
     www->type=type_buffer;
+    www->free_type=1;
 
 #if RAPTOR_DEBUG > 2
     RAPTOR_DEBUG3("Got content type '%s' (%d bytes)\n", type_buffer, len);
@@ -130,6 +156,11 @@ raptor_www_curl_init(raptor_www *www)
 #endif
 
   curl_easy_setopt(www->curl_handle, CURLOPT_ERRORBUFFER, www->error_buffer);
+
+  /* Connection timeout in seconds */
+  curl_easy_setopt(www->curl_handle, CURLOPT_CONNECTTIMEOUT,
+                   www->connection_timeout);
+  curl_easy_setopt(www->curl_handle, CURLOPT_NOSIGNAL, 1);
 }
 
 
@@ -155,12 +186,16 @@ raptor_www_curl_fetch(raptor_www *www)
   if(www->user_agent)
     curl_easy_setopt(www->curl_handle, CURLOPT_USERAGENT, www->user_agent);
 
-  /* Insert HTTP Accept: header only */
-  if(www->http_accept) {
+  if(www->http_accept)
     slist=curl_slist_append(slist, (const char*)www->http_accept);
+
+  /* ALWAYS disable curl default "Pragma: no-cache" */
+  slist=curl_slist_append(slist, "Pragma:");
+  if(www->cache_control)
+    slist=curl_slist_append(slist, (const char*)www->cache_control);
+
+  if(slist)
     curl_easy_setopt(www->curl_handle, CURLOPT_HTTPHEADER, slist);
-  }
-  
 
   /* specify URL to get */
   curl_easy_setopt(www->curl_handle, CURLOPT_URL, 
